/**
 *  Copyright (c) 2015, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the license found in the
 *  LICENSE-examples file in the root directory of this source tree.
 */

'use strict';

exports.__esModule = true;
exports['default'] = getQueryFacts;
exports.getSelectedOperation = getSelectedOperation;
exports.collectVariables = collectVariables;

var _graphql = require('graphql');

/**
 * Provided previous "queryFacts", a GraphQL schema, and a query document
 * string, return a set of facts about that query useful for GraphiQL features.
 *
 * If the query cannot be parsed, returns undefined.
 */

function getQueryFacts(prevFacts, schema, documentStr) {
  if (!documentStr) {
    return;
  }

  var documentAST = undefined;
  try {
    documentAST = _graphql.parse(documentStr);
  } catch (e) {
    return;
  }

  var variableToType = schema ? collectVariables(schema, documentAST) : null;

  // Collect operations by their names.
  var operations = [];
  documentAST.definitions.forEach(function (def) {
    if (def.kind === 'OperationDefinition') {
      operations.push(def);
    }
  });

  // Determine what the selected operation should be.
  var selectedOperation = getSelectedOperation(prevFacts, operations);

  return { variableToType: variableToType, operations: operations, selectedOperation: selectedOperation };
}

/**
 * Provided previous "queryFacts" and a list of operations, determine what the
 * next selected operation should be.
 */

function getSelectedOperation(prevFacts, operations) {
  // If there are not enough operations to bother with, return nothing.
  if (!operations || operations.length < 1) {
    return;
  }

  // If a previous selection still exists, continue to use it.
  var names = operations.map(function (op) {
    return op.name && op.name.value;
  });
  var prevSelectedOperation = prevFacts.selectedOperation;
  if (prevSelectedOperation && names.indexOf(prevSelectedOperation) !== -1) {
    return prevSelectedOperation;
  }

  // If a previous selection was the Nth operation, use the same Nth.
  if (prevSelectedOperation && prevFacts.operations) {
    var prevNames = prevFacts.operations.map(function (op) {
      return op.name && op.name.value;
    });
    var prevIndex = prevNames.indexOf(prevSelectedOperation);
    if (prevIndex && prevIndex < names.length) {
      return names[prevIndex];
    }
  }

  // Use the first operation.
  return names[0];
}

/**
 * Provided a schema and a document, produces a `variableToType` Object.
 */

function collectVariables(schema, documentAST) {
  var variableToType = Object.create(null);
  documentAST.definitions.forEach(function (definition) {
    if (definition.kind === 'OperationDefinition') {
      var variableDefinitions = definition.variableDefinitions;
      if (variableDefinitions) {
        variableDefinitions.forEach(function (_ref) {
          var variable = _ref.variable;
          var type = _ref.type;

          var inputType = _graphql.typeFromAST(schema, type);
          if (inputType) {
            variableToType[variable.name.value] = inputType;
          }
        });
      }
    }
  });
  return variableToType;
}